; This file is part of the source code for "Super Connard" V1
; Creative Commons BY-NC-SA 2013 - furrtek.org

emucheck:
  di

  call   wait_vbl

.IF VERBOSE == 1
  call   waitvbl
  ld     a,%00010001            ;Turn off screen
  ldh    ($40),a

  ;Load alphabet
  ld     hl,(:tiles_alpha*$4000)+tiles_alpha
  ld     de,$8000
  call   loadtiles_auto

  call   clearbkg
  call   clearsprites

  xor    a
  ldh    ($42),a                ;Scroll Y
  ldh    ($43),a                ;Scroll X
.ENDIF

  ;By default, we're on an emulator
  ld     a,$FF
  ld     (ISEMULATOR),a

;----------------------------------------------------------------
;Emulator detection: DMA copy
;-The DMA copy isn't instantaneous
;-Data must be the same in dest and source

  call   wait_vbl

  xor   a
  ld    ($DF00),a

  ;Initialize OAM buffer in RAM (160,168,0,0)
  ld     hl,OAMCOPY
  ld     b,40
ldoam:
  ld     a,160
  ldi    (hl),a
  ld     a,168
  ldi    (hl),a
  xor    a
  ldi    (hl),a
  ldi    (hl),a
  dec    b
  jr     nz,ldoam

  call   RAMtoOAM

  ld     a,(ISEMULATOR)
  cp     $C2            ; If the copy went too fast: emulator !
  ret    z

.IF VERBOSE == 1
  ld     de,text_passdma
  ld     hl,$9800+(2*32)+2
  ld     b,32
  call   maptext
.ENDIF

;----------------------------------------------------------------
;Emulator detection: behavior of wave RAM
;Must be cleared after wave channel read

  ld     a,$80
  ldh    ($26),a        ;APU on
  xor    a
  ldh    ($24),a        ;No routing (can't hear anything)
  ldh    ($25),a

  ld     hl,$FF30
  ld     de,mailaddr
  ld     b,16
loadwave:
  ld     a,(de)
  sub    5
  ldi    (hl),a
  inc    de
  dec    b
  jr     nz,loadwave

  ld     a,%00100000
  ldh    ($1C),a        ;CH3 volume
  ld     a,$80
  ldh    ($1A),a        ;Playback
  ldh    ($1D),a        ;Frequency
  ldh    ($1E),a        ;Initial

  ld     b,$FF		;Pause
-:
  dec    b
  jr     nz,-

  ld     hl,$FF30       ;Compare with wave RAM
  ld     de,mailaddr
  ld     b,16
cmpwave:
  ld     a,(de)
  ld     c,a
  ldi    a,(hl)
  cp     c
  jr     nz,+           ;Difference: ok
  inc    de
  dec    b
  jr     nz,cmpwave
  jp     cutch3
+:
  call   cutch3

.IF VERBOSE == 1
  ld     de,text_passwave
  ld     hl,$9800+(4*32)+2
  ld     b,32
  call   maptext
.ENDIF

  ; All tests passed: we're probably not on an emulator
  xor    a
  ld     (ISEMULATOR),a

.IF VERBOSE == 1
  ld     a,%11100100            ;BG palette
  ldh    ($47),a
  ld     a,%10010001            ;Turn screen on, BG on, sprites OFF, window OFF, tiles at $8000
  ldh    ($40),a
.ENDIF

  ret

cutch3:
  xor    a              ;Turn off CH3
  ldh    ($1A),a        ;Playback
  ldh    ($1E),a        ;Initial
  ldh    ($26),a        ;Turn off APU
  ret

mailaddr:
.db "furrtek@gmail.co"
