; This file is part of the source code for "Super Connard" V1
; Creative Commons BY-NC-SA 2013 - furrtek.org

gk_update_sprites:
;----------------------------------------------------------------
;Update sprite data
  ld     a,(SCORE)	; Food acceleration according to score (MSB, speed = 1,2 or 3 px/frame)
  srl    a
  ld     c,a
  ld     b,3		; Speed factor
  ld     d,0
-:
  ld     a,(SPEEDACC)
  add    c
  ld     (SPEEDACC),a
  jr     nc,+
  inc    d
+:
  dec    b
  jr     nz,-
  ld     a,d
  ld     (FOODSPEED),a

  ld     hl,FOODLIST
  ld     b,NUM_FOOD
-:
  ldi    a,(hl)		; Type
  cp     5
  jr     nz,+
  ;X update for poisons
  ldi    a,(hl)		; Gets Y, X back
  inc    hl		; Skip X, Xofs
  sla    a		; *2 (frequency)
  push   hl
  and    $7F
  ld     hl,(:Cosinus*$4000)+Cosinus
  rst    0
  sub    $80            ; Centering
  sra    a
  sra    a
  sra    a
  sra    a		; /16 signed
  ld     c,a
  pop    hl
  ldd    (hl),a		; X ofs, X
  dec    hl             ; Y
  ld     a,(INVINC)
  or     a
  jr     nz,+		; No need for collision check if invincible
  ; Check poison collision with hand
  ; Check Y
  ldi    a,(hl)         ; Y of food
  cp     $4A
  jr     c,++           ; Y before $4A: no collision possible
  cp     $68
  jr     nc,++          ; Y after$68: no collision possible
  ; Check X
  ld     a,(HAND_X)
  add    4
  cpl
  add    12             ; Left hitbox
  cp     (hl)
  jr     nc,++
  add    24             ; Right Hitbox
  cp     (hl)
  jr     c,++
  ; Kill !
  call   poisonhit
  dec    hl		; Return to Y for the rest...
  jr     +++
++:
  dec    hl		; Return to Y for the rest...
+:
  ;Update Y:
  ld     a,(FOODSPEED)
  ld     c,a
  ld     a,(hl)         ; Y Gravity
  cp     $90            ; Increment Y with speed if under $90 (scrolling), else inc 1 normal
  jr     nc,+
  add    c
  ld     c,a
+:
  inc    a
  cp     $E0            ; "imaginary Y" from $90 to $E0 for the blinking timing, just inc if < $E0
  jr     nz,nextfood
  ; Reinit food after timeout
+++:
  dec    hl             ; Type
  ld     a,(hl)
  and    7		; Clears flags
  cp     5
  jr     nz,+
  ld     a,(POISONNB)   ; One poison less on the screen
  dec    a
  ld     (POISONNB),a
+:
  call   get_random
  ld     a,(RAND8)      ; Type random, if > 200: poison, else 0 to 4 for normal food
  cp     200
  jr     c,+++
  ;See if we can set up a poison if there's not already 2
  ld     a,(POISONNB)
  cp     2
  jr     z,+++		; Already too much
  inc    a
  ld     (POISONNB),a
  ld     a,5		; Setup poison
  ldi    (hl),a		; Store type
  xor    a
  ldi    (hl),a         ; Y
  call   getrandx
  ldi    (hl),a 	; X, Xofs
  inc    hl             ; Type
  jr     ++
+++:
  swap   a		; Normal food
  and    7
  cp     5
  jr     c,+
  sub    4
+:
  ldi    (hl),a		; Store type
  call   get_random
  ld     a,(RAND8)      ; Random Y (0~15)
  swap   a
  and    $F
  ldi    (hl),a
  call   getrandx
  ldi    (hl),a         ; X
  xor    a
  ldi    (hl),a		; X ofs, #
  jr     ++             ; Next...
nextfood:
  ldi    (hl),a         ; MaJ Y
  inc    hl             ; Skip X
  inc    hl		; Skip X ofs
++:
  dec    b
  jp     nz,-
  ret
  
  
getrandx:
  call   get_random
  ld     a,(RAND8)      ; Random X (20~160-32)
  cp     160-32
  jr     c,+
  sub    160-32
+:
  add    20
  ret
  

poisonhit:
  push   hl
  push   bc
  ld     hl,fx_grabpoison
  call   playsfx
  ld     hl,LIVES
  dec    (hl)    	; Gameover is done in vbl
  ld     a,135		; Life tile
  ld     c,$4A
  call   update_lives
  ;16bit BCD dec -400
  ld     hl,SCORE       ; HL=(SCORE)
  ldi    a,(hl)
  ld     l,(hl)
  cp     4
  jr     nc,subscore
  ld     hl,0		; Limit to 0 if <400
  jr     +
subscore:
  sub    4		; -400
  daa
  ld     h,a
+:
  ld     a,h            ; (SCORE)=HL
  ld     (SCORE),a
  ld     a,l
  ld     (SCORE+1),a
  ld     c,75
  call   disp_score     ; Update score display
  call   gen_passwd	; Regenerate password for pause mode with current score
  ld     hl,$9800+10    ; Display pass
  ld     de,PASSWD_GEN
  ld     b,$D0+1-65
  call   maptext
  ld     a,$20          ; Close hand when poison hit (it hurts !)
  ld     (CLENCH),a
  call   fistiniere
  ld     a,$FF     	; Invincible for a moment
  ld     (INVINC),a
  pop    bc
  pop    hl
  ret


fistiniere:
  ld     bc,$0505
  ld     a,37
  ld     hl,$9800+(32*8)       ; Map closed hand
  call   mapinc_w
  ret


gk_render_sprites:
;----------------------------------------------------------------
;Sprite rendering

  ld     hl,OAMCOPY
  ld     de,FOODLIST
  ld     b,NUM_FOOD
-:
  ld     a,(de)         ; Type read
  bit    7,a
  jr     z,+            ; Grabbed or not ?

; Grabbed, display sprite in hand:
  and    $3F

  ld     c,a            ; Save A (food #)
  inc    de		; Y
  inc    de             ; X
  inc    de             ; X ofs

  ;Left sprite
  ld     a,$5C
  ld     (SPR16_Y),a    ; Y position
  ld     a,(HAND_X)     ; X position
  add    4
  cpl
  add    24
  ld     (SPR16_X),a
  ld     a,c            ; Restore A
  sla    a              ; *4 (4 tiles/food)
  sla    a
  add    136            ; First food (apple)
  ld     (SPR16_N),a
  xor    a
  ld     (SPR16_A),a
  
  call   make8x16sprite

  jr     ++

+:

; Not grabbed, normal display:
  and    $3F
  ld     c,a            ; Save A (food #)
  inc    de

  ;Y handling (also used as timer after $90)
  ld     a,(de)         ; Read Y
  cp     $90            ; Limit to $90 and keep runing (optimisation for reinit timing)
  jr     c,+            ; If we're under $90, make the sprite scroll normally
  dec    de		; Type and flags
  ld     a,(de)
  inc    de
  bit    6,a            ; Ground hit flag ?
  jr     nz,++++
  set    6,a            ; Not yet, set flag and play sound
  dec    de
  ld     (de),a
  inc    de
  push   hl
  ld     hl,fxground
  call   playsfx
  pop    hl
++++:
  ld     a,(de)         ; Read Y
  cp     $B0
  jr     c,+++          ; If we're under $B0, keep the sprite on the ground
  and    4              ; Blinking speed
  jr     nz,+++         ; If we're over, make blink
  inc    de
  inc    de
  jr     ++             ; Blinking: hidden sprite, skip and go to next...
+++:
  ld     a,$90
+:
  ld     (SPR16_Y),a
  inc    de
  ld     a,(de)         ; Read X
  push   bc
  ld     c,a
  inc    de
  ld     a,(de)         ; Read X ofs
  add    c
  ld     (SPR16_X),a
  pop    bc
  ld     a,c            ; Restore A
  sla    a              ; *4 (4 tiles/food)
  sla    a
  add    136            ; First food (apple)
  ld     (SPR16_N),a
  xor    a
  ld     (SPR16_A),a

  call   make8x16sprite

++:
  inc    de
  dec    b
  jr     nz,-
  ret



gk_inc_score:
;----------------------------------------------------------------
;Progressive inc of score
  ld     a,(TOADD)
  or     a
  jr     z, +                   ;Nothing to add

  call   sh_incscore

  ld     c,75
  call   disp_score             ;Update score display
+:
  ret


anim_paysans:
;----------------------------------------------------------------
;Update people animation
  call   get_random

  ld     d,a
  ld     a,62
  bit    7,d
  jr     z,+
  ld     a,68
+:
  ld     bc,$0202
  ld     hl,$9C00+3
  call   mapinc_w
  ld     a,62
  bit    6,d
  jr     z,+
  ld     a,68
+:
  ld     bc,$0202
  ld     hl,$9C00+7
  call   mapinc_w
  ld     a,62
  bit    5,d
  jr     z,+
  ld     a,68
+:
  ld     bc,$0202
  ld     hl,$9C00+11
  call   mapinc_w
  ld     a,62
  bit    4,d
  jr     z,+
  ld     a,68
+:
  ld     bc,$0202
  ld     hl,$9C00+15
  call   mapinc_w
  ret


checkgrab:
;----------------------------------------------------------------
;Grab check
  ld     hl,FOODLIST
  ld     b,NUM_FOOD
-:
  ld     a,(hl)
  bit    7,a
  jr     z,+
  inc    hl
  inc    hl
  jr     ++             ; Already grabbed, next...
+:
  ldi    a,(hl)
  cp     5
  jr     nz,+		; A poison (we can while being invincible), next...
  inc    hl
  jr     ++
+:
  ; Check Y
  ldi    a,(hl)         ; Y of food
  cp     $50
  jr     c,++           ; Y before $50: no grab possible
  cp     $68
  jr     nc,++          ; Y after $68: no grab possible
  ; Check X
  ld     a,(HAND_X)
  add    4
  cpl
  add    12             ; Left hitbox
  cp     (hl)
  jr     nc,++
  add    24             ; Right hitbox
  cp     (hl)
  jr     c,++
  dec    hl             ; Y
  dec    hl             ; Type
  push   hl             ; 0

  ld     a,(hl)
  ld     d,0
  ld     e,a
  ld     hl,kim_scoretable
  add    hl,de
  ld     a,(TOADD)
  add    (hl)           ; Player inc scores !
  ld     (TOADD),a
  
  ld     hl,fx_grabok
  call   playsfx

  pop    hl             ; 0
  ld     a,(hl)
  set    7,a            ; Grabbed bit set
  ldi    (hl),a
  ld     a,$C0          ; Timeout hand disappear in Y position (relative to $E0)
  ld     (hl),a

  ret
++:
  inc    hl
  inc    hl
  dec    b
  jp     nz,-

  ld     hl,fx_grabmiss
  call   playsfx
  ret
  
kim_scoretable:
.db 40,30,20,15,10	; See text.asm
