; This file is part of the source code for "Super Connard" V1
; Creative Commons BY-NC-SA 2013 - furrtek.org

gr_update_road:
; Make scroll speed for BG from curve and car speed
  ld     a,(ROAD_X)
  bit    7,a
  push   af             ; Save flags to avoid doing bit again
  jr     nz,+
  cpl                   ; Signed to unsigned
+:
  swap   a
  and    7              ; /16
  ld     e,a
  ld     a,(ROADSPEED)
  ld     h,a
  call   mul8_8
  ld     b,h
  pop    af		; Flags
  jr     z,+
  ld     a,b
  dec    a              ; Unsigned to signed, stupid ain't it ?
  cpl
  ld     b,a
+:
  ld     a,(BKG_X)
  add    b
  ld     (BKG_X),a

; Make offset table for the road's curve
  ld     hl,roadtable
  ld     de,ROAD_OFFSETS
  ld     b,58           ; Lines to do
-:
  ldi    a,(hl)
  push   hl
  push   de
  ld     h,a            ; Transforms 0~FF to FF~0~FF
  ld     a,(ROAD_X)
  bit    7,a
  push   af		; Save flags
  jr     nz,+
  cpl                   ; Signed to unsigned
+:
  sla    a
  ld     e,a
  call   mul8_8         ; ROAD_X (curve amplitude) * table
  pop    af		; Restore flags
  ld     a,h            ; Keep MSB
  jr     nz,+
  cpl                   ; Unsigned to signed
+:
  pop    de
  pop    hl
  ld     (de),a		; Place in RAM
  inc    de
  inc    de
  dec    b
  jr     nz,-

  ;Make palette flags for the road's edges (interleaved with curve offsets)
  ld     de,ROAD_OFFSETS+1
  ld     b,58           ; Lines to do
-:
  ld     a,(PALSPEED)
  srl    a
  srl    a
  srl    a
  ld     l,a
  ld     a,(PALACC)
  add    l
  ld     (PALACC),a
  and    $10
  ld     a,%11100100            ;BG palette
  jr     z,++
  ld     a,%11100000            ;Alt. BG palette
++:
  ld     (de),a

  ld     hl,PALSPEED
  ld     a,(hl)
  cp     1
  jr     z,+
  dec    (hl)

  inc    de
  inc    de
  dec    b
  jr     nz,-



  ;Delta update for the curve change
  ld     hl,ROAD_GOAL_DELTA
  ldi    a,(hl)                 ; =0? 16bits
  or     (hl)
  jr     nz,+
  ;Delta a zero, maj nouveau goal
  call   get_random
  ld     a,(RAND8)
  swap   a
  and    $7
  ld     hl,road_goaltable	; The curve's goal comes from a 8 entry table
  rst    $0
  ld     (ROAD_GOAL),a
  ;Nouveau delta
  call   get_random
  ld     a,(RAND8)
  and    7
  inc    a
  ld     d,a
  ld     e,$FF
  jr     ++
+:
  ld     hl,ROAD_GOAL_DELTA
  ld     d,(hl)                 ; (HL)-- 16bits
  inc    hl
  ld     e,(hl)
  ld     a,(ROADSPEED)  ;HHHHLLLL
  swap   a		;LLLLHHHH
  srl	 a              ;0LLLLHHH
  and    7
  jr     z,+++
  ld     b,a
-:
  dec    de
  ld     a,d
  or     e
  jr     z,++
  dec    b
  jr     nz,-
++:
  ld     hl,ROAD_GOAL_DELTA
  ld     (hl),d
  inc    hl
  ld     (hl),e
+++:

  ;Update road_x with goal
  ld     a,(ROAD_X)
  ld     b,a
  ld     a,(ROAD_GOAL)
  cp     b
  jr     z,+			;Road_x and goal the same, do nothing
  sub    b
  ld     a,b
  jr     c,++                   ;Inferior
  inc    a
  jr     +++                    ;Superior
++:
  dec    a
+++:
  ld     (ROAD_X),a
+:
  ret


gr_collision:
  ld     a,(ANIMAL_TYPE)
  bit    7,a
  ret     nz			;Already flying, no check needed

  ld     a,(ANIMAL_Y)           ;Check hitbox 4 edges
  cp     75
  ret    c                      ;Too high, no collision
  ld     a,(ANIMAL_Y)
  cp     80
  ret    nc                     ;Too low, car bypassed, no collision
  ld     a,(CAR_X)
  ld     b,a
  ld     a,(ANIMAL_X)
  add    10
  cp     b			;Right animal edge vs left car edge
  ret    c			;Too far to the left, no collision
  ld     c,b
  ld     b,a                    ;Stupid exchange tricks
  ld     a,c			;A=CAR_X B=ANIMAL_X
  add    28
  cp     b			;Left animal edge vs right car edge
  ret    c                      ;Too far to the right, no collision

  ;Collision !
  ;Horizontal flying speed made from impact side (animal/car X difference)
  ld     a,c
  sub    b
  sra    a
  sra    a
  add    4			;Alignment
  ld     (ANIMAL_X_THROW),a

  ld     hl,fx_hit
  call   playsfx

  ld     a,(ANIMAL_TYPE)	;Did we hit an animal or a cement block ?
  cp     GR_TYPE_CEMENT
  jr     nz,+

  ;Cement block:
  ld     hl,fx_grabpoison
  call   playsfx
  ld     hl,LIVES
  dec    (hl)    	; Gameover is done in gk_update_lives
  ld     a,GR_T_LIFE	; Life tile
  ld     c,GR_T_BLANK
  call   update_lives
  xor    a            	; Set speed to zero
  ld     (ROADSPEED),a
  ;16bit BCD dec -100
  ld     hl,SCORE       ; HL=(SCORE)
  ldi    a,(hl)
  ld     l,(hl)
  cp     1
  jr     nc,++
  ld     hl,0		; Limit to 0 if <100
  jr     +++
++:
  sub    1		; -100
  daa
  ld     h,a
+++:
  ld     a,h            ; (SCORE)=HL
  ld     (SCORE),a
  ld     a,l
  ld     (SCORE+1),a
  ld     c,GR_T_BIGNUMS
  call   disp_score     ; Update score display
  call   gen_passwd	; Regen password for pause mode with current score
  ld     hl,$9800+10    ; Display score
  ld     de,PASSWD_GEN
  ld     b,$82
  call   maptext
  jr     ++
+:
  ; Animal:
  ld     d,0
  ld     e,a
  ld     hl,road_scoretable
  add    hl,de
  ld     a,(TOADD)
  add    (hl)           	; Player inc scores !
  ld     (TOADD),a

++:
  ; Vertical flying speed (depends on car speed)
  ld     a,(ROADSPEED)
  swap   a	;32107654
  sla    a      ;2107654-
  and    $18    ;---76---
  add    5
  ld     (ANIMAL_Y_THROW),a

  ld     hl,ANIMAL_TYPE
  set    7,(hl)			;Hit flag
  ;Convert the Y in 7.1 offset +90, in an intenger offset 0 for the flying rendering code
  ld     a,(ANIMAL_Y)
  srl	 a
  add    98
  ld     (ANIMAL_Y),a

  ret


reset_animal:
  call   get_random
  ld     a,(RAND8)
  swap   a
  srl	 a
  srl    a
  ld     (ANIMAL_XVIS),a	; X position on the road (0~63)
  and    3
  ld     (ANIMAL_TYPE),a
  xor    a
  ld     (ANIMAL_Y),a
  ld     a,170
  ld     (ANIMAL_X),a
  ld     a,$40
  ld     (APPEAR_DELTA),a
  xor    a                      ; For over there (1)
  ret


gr_inc_score:
;----------------------------------------------------------------
;Instantaneous score inc (we don't have time to progressively inc, poor Gameboy :( )
  ld     a,(TOADD)
  or     a
  ret    z
  call   upd_score

  ld     c,GR_T_BIGNUMS
  call   disp_score             ;Update score display
  
  xor    a
  ld     (TOADD),a
  ret




gr_update_sprites:
;----------------------------------------------------------------
;Update sprite data

;Animal update

;Appear delta update
  ld     a,(ROADSPEED)
  or     a
  jr     z,++
  swap   a
  and    $F
  ld     b,a
  ld     a,(APPEAR_DELTA)
  cp     b
  jr     nc,+
  ld     b,a		; Limit to zero
+:
  sub    b
  ld     (APPEAR_DELTA),a
++:

  ld     a,(APPEAR_DELTA)
  or     a
  jp     nz,+++		; Nothing to do while delta > 0

  ld     a,(ANIMAL_TYPE)
  bit    7,a
  jr     nz,++		; Is flying, go to throw code (++)

  ;On the road, make Y (exponential inc to simulate perspective)
  ld     a,(ROADSPEED)
  or     a
  jr     z,+++
  rlc    a
  rlc    a
  and    3		; /64 -> 0~3
  xor    3
  ld     c,a
  inc    c

  ld     a,(ANIMAL_Y)
  ld     b,a
  swap   a
  and    $F		; /16
-:
  srl	 a
  dec    c
  jr     nz,-
  inc    a              ; To avoid getting stuck at 0
  add    b              ; Y=(Y/16/c)+1
  cp     120
  call   nc,reset_animal

  ; Calcul X
  ld     (ANIMAL_Y),a	; For here (1)
  ld     b,a
  ;srl    a
  ;sla    a
  and    $FE
  ld     hl,ROAD_OFFSETS
  rst    0
  cpl
  add    78		; Left of the road, "RoadX"
  ld     c,a

  ld     a,b		; ANIMAL_Y
  sla    a		; Since Y is 0~150 => 105~255 to get XVIS at max
  ld     e,a
  ld     a,(ANIMAL_XVIS)
  ld     h,a
  call   mul8_8
  ld     a,(ANIMAL_XVIS)
  bit    5,a
  ld     a,h		; MSB
  jr     z,+            ; Unsigned -> signed, cheat (bit 5 instead of 7 seeing the range of ANIMAL_XVIS)
  dec    a
  cpl
+:
  add    c
  ld     (ANIMAL_X),a
  jr     +++

++:
  ;Throw: Make the animal fly
  ld     a,(ANIMAL_Y_THROW)
  dec    a                              ;Gravity
  ld     (ANIMAL_Y_THROW),a
  sra    a
  sra    a
  ld     b,a
  ld     a,(ANIMAL_Y)
  sub    b
  cp     150                            ;Exit bottom of screen
  jr     c,+
  call   reset_animal
  jr     +++
+:
  ld     (ANIMAL_Y),a

  ld     a,(ANIMAL_X_THROW)
  ld     b,a
  ld     a,(ANIMAL_X)
  sub    b
  cp     160
  jr     nc,+
  cp     4
  jr     c,+
  jr     ++++
+:
  call   reset_animal			;Exit on the left or right of the screen
  jr     +++
++++:
  ld     (ANIMAL_X),a

+++:
;Update car force
  ld     hl,CAR_PUSH
  
  ld     a,(ROADSPEED)  ; B = Times to add to CAR_X_LOW from ROADSPEED (0 to 5)
  rlc    a
  rlc    a
  and    3
  or     a
  jr     z,++
  ld     b,a

  ld     a,(ROAD_X)	; C = move direction (-1,0,+1)
  sub    $80            ; 0~$FF -> $80~$0~$FF
  sra    a
  sra    a
  sra    a
  sra    a
  sra    a
  ld     c,a

  ld     d,0
-:
  ld     a,(CAR_X_LOW)
  add    c
  ld     (CAR_X_LOW),a
  ld     a,d
  add    c
  ld     d,a
  dec    b
  jr     nz,-

++:

  ld     a,d
  bit    7,a
  jr     z,+
  dec    a		; NEG :/
  cpl
+:
  ld     (hl),a		; CAR_PUSH

  ld     a,(CAR_X_LOW)
  bit    7,a
  jr     z,+
  ;Negative overflow:
  and    7
  ld     (CAR_X_LOW),a
  ld     a,(CAR_X)
  dec    a
  cp     8
  jr     c,++
  ld     (CAR_X),a
  jr     ++
+:
  bit    3,a
  jr     z,++
  ;Positive overflow
  and    7
  ld     (CAR_X_LOW),a
  ld     a,(CAR_X)
  inc    a
  cp     160-28
  jr     nc,++
  ld     (CAR_X),a
++:
  ret
  


gr_render_sprites:
  ; Car (32x16)
  ld     hl,OAMCOPY

  ld     a,140
  ld     (SPR16_Y),a
  ld     a,(CAR_X)
  ld     (SPR16_X),a
  ld     a,GR_T_CAR1
  ld     (SPR16_N),a
  xor    a
  ld     (SPR16_A),a
  call   make8x16sprite

  ld     a,140
  ld     (SPR16_Y),a
  ld     a,(CAR_X)
  add    16
  ld     (SPR16_X),a
  ld     a,GR_T_CAR2
  ld     (SPR16_N),a
  xor    a
  ld     (SPR16_A),a
  call   make8x16sprite

  ;Drift effect
  ld     a,(DRIFTING)
  or     a
  jr     z,+

  ld     a,(VBLC) 	; Animation (oscillating X)
  and    4
  ld     b,a

  ;Left drift fx
  ld     a,148
  ldi    (hl),a		;Y
  ld     a,(CAR_X)
  sub    10
  add    b
  ldi    (hl),a         ;X
  ld     a,GR_T_DRIFT
  ldi    (hl),a         ;N
  ld     a,%00100000
  ldi    (hl),a		;Attribs

  ;Right drift fx
  ld     a,148
  ldi    (hl),a		;Y
  ld     a,(CAR_X)
  add    30
  add    b
  ldi    (hl),a         ;X
  ld     a,GR_T_DRIFT
  ldi    (hl),a         ;N
  xor    a
  ldi    (hl),a		;Attribs
+:

  ;Animal
  ld     a,(APPEAR_DELTA)
  or     a
  jp     nz,++		; Nothing to do while delta > 0

  ld     a,(ANIMAL_TYPE)
  bit    7,a
  ld     a,(ANIMAL_Y)
  jr     nz,+
  srl	 a
  add    98
+:
  ld     (SPR16_Y),a
  ld     a,(ANIMAL_X)
  ld     (SPR16_X),a
  ld     a,(ANIMAL_TYPE)
  and    3
  sla	 a
  sla    a
  add    GR_T_CONE	; Base tile number
  ld     (SPR16_N),a
  ld     a,(ANIMAL_TYPE)
  bit    7,a
  ld     a,0
  jr     z,+
  ld     a,%01000000	; Flip Y if flying
+:
  ld     (SPR16_A),a
  call   make8x16sprite
++:

  ret


gr_map_strip:
  ld     a,(STRIPE)		; Road stripes animation
  and    3
  ld     b,a
  sla	 a     ;*2
  sla    a     ;*4
  sla    a     ;*8
  sub    b     ;- = *7
  add    GR_T_STRIPES
  ld     c,a

  ld     b,7
  ld     hl,$9800+(11*32)+9
  ld     de,32
-:
  call   wait_write
  ld     (hl),c
  inc    c
  add    hl,de
  dec    b
  jr     nz,-
  ret


gr_sound:
  ld     a,(ROADSPEED)		; Engine sound from speed + vibrato
  ld     b,a
  ld     a,(VBLC)
  and    2
  swap   a
  add    b
  sla    a
  sla    a
  ldh    ($18),a
  ld     a,(ROADSPEED)
  rlc    a
  rlc    a
  and    3
  ;or     %00000010
  ldh    ($19),a

  ld     a,(DRIFTING)
  or     a
  jr     z,+
  ld     a,(VBLC)
  and    8
  jr     z,+
  ;Drifting sound
  ld     a,(VBLC)
  and    9
  cp     9
  jr     nz,+
  ld     hl,fx_drift
  call   playsfx
+:
  ret

gr_gameover:
  ld     de,Setup_gameover
  call   Setup_exit_white
  ret

road_scoretable:
.db 10  	; Cone
.db 20          ; Fox
.db 40  	; Ronflex

.include "roadtable.asm"

; Angles tables for road
road_goaltable:
.db $20,$30,$50,$7F,$80,$90,$B0,$D0
