; This file is part of the source code for "Super Connard" V1
; Creative Commons BY-NC-SA 2013 - furrtek.org

;----------------------------------------------------------------
;Password generation from score, time, lives and game number

gen_passwd:
  ld     hl,PASSWD_BS

  ;Build bitstream	 MMMMMMPP PPPPPPPP PPPPPPSS SSSSSVVV JJCCCCC
  ld     a,(TIME_MIN)
  and    $3F
  sla	 a		;0MMMMMM0
  sla	 a		;MMMMMM00
  ld     b,a
  ld     a,(SCORE)	;PPPPpppp
  swap   a              ;ppppPPPP
  srl    a		;0ppppPPP
  srl    a              ;00ppppPP
  and    3              ;000000PP
  or     b		;MMMMMMPP
  ldi    (hl),a		;Byte #1

  ld     a,(SCORE)	;PPPPPPPP
  sla    a
  sla    a		;PPPPPP00
  ld     b,a
  ld     a,(SCORE+1)    ;Pppppppp
  swap   a              ;ppppPppp
  srl    a		;0ppppPpp
  srl    a              ;00ppppPp
  and    3              ;000000Pp
  or     b              ;PPPPPPpp
  ldi    (hl),a		;Byte #2
  
  ld     a,(SCORE+1)	;pppppppp
  sla    a
  sla    a		;pppppp00
  ld     b,a
  ld     a,(TIME_SEC)	;0SSSSSSS
  swap   a		;SSSS0SSS
  srl	 a		;0SSSS0SS
  and    3              ;000000SS
  or     b              ;ppppppSS
  ldi    (hl),a		;Byte #3

  ld     a,(TIME_SEC)	;0SSSSSSS
  and    $7F
  sla    a
  sla    a
  sla    a		;SSSSS000
  ld     b,a
  ld     a,(LIVES)	;00000VVV
  or     b		;SSSSSVVV
  ldi    (hl),a		;Byte #4

  ld     a,(GAME_FLAGS) ;000000JJ
  and    3
  swap   a		;00JJ0000
  sla    a
  sla    a		;JJ000000
  ; Check here
  ldi    (hl),a		;Byte #5
  
  ;Bitstream checksum (sum of the 4 first bytes AND $3F)
  ld     hl,PASSWD_BS
  ld     b,4
  xor    a
-:
  add    (hl)
  inc    hl
  dec    b
  jr     nz,-
  and    $3F
  or     (hl)
  ld     (hl),a
  
  call   pass_xor
  
  ;Convert to 5-bit blocks
  ld     hl,PASSWD_PTR
  ld     de,PASSWD_GEN
  rst    $10
  ld     hl,PASSWD_BS

  ld     b,40		;40 bits
  ld     c,5            ;5 bits per 5 bits
  ld     e,0            ;buffer
-:
  ld     a,b
  and    7
  jr     nz,+
  ldi    a,(hl)
  ld     d,a
+:
  sla    e
  sla    d
  ld     a,0
  adc    e
  ld     e,a
  dec    c
  jr     nz,+
  push   hl
  push   de
  ld     hl,PASSWD_PTR
  ldi    a,(hl)
  ld     l,(hl)
  ld     h,a
  ld     (hl),e
  inc    hl
  push   hl
  pop    de
  ld     hl,PASSWD_PTR
  ld     a,d
  ldi    (hl),a
  ld     (hl),e
  pop    de
  pop    hl
  ld     c,5
  ld     e,0
+:
  dec    b
  jr     nz,-

  ;Convert to alphabet
  ld     hl,PASSWD_GEN
  ld     b,8
-:
  ld     a,(hl)
  cp     26                     ;Offset alphabet/numbers
  jr     c,+
  sub    43
+:
  add    65
  ldi    (hl),a
  dec    b
  jr     nz,-

  ld     a,$FF
  ld     (hl),a			;End of string

  ret



check_passwd:
  ; Check if PASSWD_DISP is valid, sets up the game and returns 1 if so, 0 if not

  ; Check length
  ld     hl,PASSWD_DISP
  ld     b,8
-:
  ldi    a,(hl)
  cp     $FF
  jr     z,+
  dec    b
  jr     nz,-
+:
  xor    a
  cp     b
  ret    nz			;Less than 8 chars: bad

  ; Convert du binary
  ld     hl,PASSWD_DISP
  ld     b,8
-:
  ld     a,(hl)
  cp     $39                    ;Offset alphabet/numbers
  jr     c,+
  sub    43
+:
  sub    22
  ldi    (hl),a
  dec    b
  jr     nz,-
  
  ; Check characters, if one is > "5" (>31), bad code
  ld     hl,PASSWD_DISP
  ld     b,8
-:
  ldi    a,(hl)
  cp     32
  jr     nc,+
  dec    b
  jr     nz,-
+:
  xor    a
  cp     b
  ret    nz			;Bad

  ; Convert to bytes (bitstream)
  ld     hl,PASSWD_PTR
  ld     de,PASSWD_BS
  rst    $10
  ld     hl,PASSWD_DISP

  ld     b,40		;40 bits
  ld     c,0            ;5 bits per 5 bits
  ld     e,0            ;buffer
-:
  ld     a,c
  or     a
  jr     nz,+
  ldi    a,(hl)		;Load new character
  sla    a
  sla    a
  sla    a
  ld     d,a
  ld     c,5
+:
  sla    e
  sla    d
  ld     a,0
  adc    e
  ld     e,a
  dec    c
  dec    b
  ld     a,b
  and    7
  jr     z,+
  jr     -
+:
  push   hl
  push   de
  ld     hl,PASSWD_PTR	;Could be done in an easier way probably...
  ldi    a,(hl)
  ld     l,(hl)
  ld     h,a
  ld     (hl),e
  inc    hl
  push   hl
  pop    de
  ld     hl,PASSWD_PTR
  ld     a,d
  ldi    (hl),a
  ld     (hl),e
  pop    de
  pop    hl
  ld     e,0
  ld     a,b
  or     a
  jr     nz,-

  call   pass_xor

  ;Checksum compare
  ld     hl,PASSWD_BS
  ld     b,4
  xor    a
-:
  add    (hl)
  inc    hl
  dec    b
  jr     nz,-
  and    $3F
  ld     b,a
  ld     a,(hl)
  and    $3F
  cp     b
  ld     a,0
  ret    nz			;Bad

  ld     hl,PASSWD_BS
  
  ;Splits into appropriate variables	 MMMMMMPP PPPPPPPP PPPPPPSS SSSSSVVV JJCCCCCC
  ld     a,(hl)      	;MMMMMMPP
  srl	 a              ;0MMMMMMP
  srl    a              ;00MMMMMM
  call   pass_checkbcd
  cp     $29+1
  jr     nc,pass_bad
  ld     (TIME_MIN),a

  ldi    a,(hl)         ;MMMMMMPP
  rrc	 a              ;PMMMMMMP
  rrc    a              ;PPMMMMMM
  and    $C0            ;PP000000
  ld     b,a
  ld     a,(hl)         ;PPPPPPPP
  srl    a              ;0PPPPPPP
  srl    a              ;00PPPPPP
  or     b              ;PPPPPPPP
  call   pass_checkbcd
  ld     (SCORE),a

  ldi    a,(hl)		;PPPPPPpp
  rrc    a		;pPPPPPPp
  rrc    a              ;ppPPPPPP
  and    $C0            ;pp000000
  ld     b,a
  ld     a,(hl)		;PPPPPPSS
  srl    a              ;0PPPPPPS
  srl    a              ;00PPPPPP
  or     b              ;ppPPPPPP
  call   pass_checkbcd
  ld     (SCORE+1),a

  ldi    a,(hl)         ;PPPPPPSS
  swap   a		;PPSSPPPP
  sla    a		;PSSPPPPP
  and    $60		;0SS00000
  ld     b,a
  ld     a,(hl)		;SSSSSVVV
  srl    a		;0SSSSSVV
  srl    a              ;00SSSSSV
  srl    a              ;000SSSSS
  or     b              ;0SSSSSSS
  call   pass_checkbcd
  cp     $59+1
  jr     nc,pass_bad
  ld     (TIME_SEC),a

  ldi    a,(hl)		;SSSSSVVV
  and    7
  jr     z,pass_nolife
  cp     4+1
  jr     nc,pass_bad
  ld     (LIVES),a

  ld     a,(hl)		;JJCCCCCC
  rlc    a              ;JCCCCCCJ
  rlc    a              ;CCCCCCJJ
  and    3
  cp     2+1
  jr     nc,pass_bad

  ld     hl,GAME_FLAGS
  res    7,(hl)

  ld     hl,pass_gamestart
  ld     d,0
  sla    a
  ld     e,a
  add    hl,de
  ldi    a,(hl)
  ld     d,(hl)
  ld     e,a
  call   Setup_exit_white

  ld     a,1
  ret
  
pass_bad:
  xor    a
  ret
  
pass_nolife:
  ld     a,2
  ret


pass_gamestart:
.dw Setup_gamekim,Setup_gameroad,Setup_gamecamp


pass_xor:
  ;Xor bitstream with serial
  ld     hl,serial_ptr
  ld     a,(hl)
  add    64
  ld     d,a
  inc    hl
  ld     e,(hl)

  ld     hl,PASSWD_BS
  ld     a,(hl)
  xor    e
  ldi    (hl),a
  ld     a,(hl)
  xor    e
  ldi    (hl),a
  ld     a,(hl)
  xor    e
  ldi    (hl),a
  ld     a,(hl)
  xor    d
  ldi    (hl),a
  ld     a,(hl)
  xor    e
  ldi    (hl),a
  ret
  
  
pass_checkbcd:
  ld     c,a
  and    $F
  cp     $A
  jr     nc,+
  ld     a,c
  swap   a
  and    $F
  cp     $A
  jr     nc,+
  ld     a,c
  ret		; BCD ok

+:
  xor    a	; Trick to skip the ret in check_passwd and ret with a=0 to say bad passwd
  inc    sp
  inc    sp
  ret
