; This file is part of the source code for "Super Connard" V1
; Creative Commons BY-NC-SA 2013 - furrtek.org

ivars:
  ld     a,(BMODE)		;4 lives in normal mode, only 1 in bastard mode
  or     a
  ld     a,1
  jr     nz,+
  add    3
+:
  ld     (LIVES),a
  xor    a
  ld     (SCORE),a
  ld     (SCORE+1),a
  ld     (TIME_SEC),a
  ld     (TIME_MIN),a
  ret

intro_shared:
  ld     de,$9800+(32*19)+1
  ld     hl,Scroll_MapPtr
  rst    $10

  xor    a
  ldh    ($42),a                ;Scroll Y
  ld     (BKG_Y),a
  ldh    ($43),a                ;Scroll X

  ld     a,0
  ldh    ($4A),a                ;Window Y
  ld     a,104
  ldh    ($4B),a                ;Window X

  ld     a,%11111111
  ldh    ($47),a                ;Bg palette

  ld     a,%11110001            ;Turn on screen, BG on, sprites OFF, window ON, tiles at $8000, window map at $9C00
  ldh    ($40),a

  ld     a,%00011000            ;Vblank+RLI
  ldh    ($41),a
  ld     a,%00000011
  ldh    ($FF),a
  ret


enter_pause:
  ld     a,%01010000            ;Bg palette
  ldh    ($47),a
  ld     a,%11100100            ;SPR0 palette
  ldh    ($48),a
  call   pause_sound
  ld     a,Sot_F_Play | Sot_F_Turnoff
  ld     (Sot_Flags),a
  ld     de,text_pause		;Display "PAUSE"
  ld     hl,$9840+15
  call   maptext
  ; Display continue/quit instructions
  ld     hl,OAMCOPY
  ld     bc,$A0
  call   clear
  
  ld     hl,OAMCOPY
  ld     bc,$0A48
  ld     d,$30
  ld     a,(SPR_PAUSEBAN)
  ld     e,a
  call   map_sprban
  ld     bc,$0768
  ld     d,$3C
  call   map_sprban

  ret

  
check_zero_lives:
  ld     a,(LIVES)		;Game over ?
  or     a
  ret    nz
  ; Epic fail !
  xor    a			;Shuts up the gameroad sound
  ldh    ($17),a
  ld     (WON),a
  ld     a,%100			;Don't call enter_pause, just freeze everything
  ld     (PAUSE_FLAG),a
  ld     de,music_lose
  ld     hl,Sot_Music
  rst    $10
  call   setupmusic
  ld     a,Sot_F_Play
  ld     (Sot_Flags),a
  call   waitforsongend
  ld     de,Setup_gameover
  call   Setup_exit_white
  ret

;B=width, C=Y, D=X, E=#
map_sprban:
-:
  ld     a,c
  ldi    (hl),a		;Y
  ld     a,d
  ldi    (hl),a		;X
  add    8
  ld     d,a
  ld     a,e
  ldi    (hl),a		;N
  inc    a
  inc    a
  ld     e,a
  xor    a
  ldi    (hl),a		;A
  dec    b
  jr     nz,-
  ret

exit_pause:
  ;call   wait_vbl
  call   pause_sound
  ld     a,Sot_F_Play
  ld     (Sot_Flags),a
  ld     hl,$9840+15		;Erase "PAUSE"
  ld     c,5
  ld     a,b
-:
  call   wait_write
  ldi    (hl),a
  dec    c
  jr     nz,-
  ld     a,%11100100            ;BG palette
  ldh    ($47),a
  ret
  
  
pause_b_handler:
  ld     a,(PAUSE_FLAG)		;Ignore quit on B button press if not paused
  bit    0,a
  ret    z
  ld     a,(JOYP_ACTIVE)
  bit    1,a
  ret    z                      ;B button
  ld     a,(Sot_Flags)		;Wait for the pause jingle to be played before reading B
  and    Sot_F_Play
  ret    nz
  ;Quit game if paused, B press and jingle played
  ld     de,Setup_gameover
  jp     Setup_exit_white
  ret

inc_score:
  call   check_win

  ld     a,l
  add    1			;INC doesn't effect the carry, it's needed for DAA !
  daa
  ld     l,a
  jr     nc,+
  ld     a,h
  add    1
  daa
  ld     h,a
+:

  ld     a,h                    ;(SCORE)=HL
  ld     (SCORE),a
  ld     a,l
  ld     (SCORE+1),a
  ret
  
  
upd_score:
  call   check_win

  ld     a,(TOADD)
  call   bin2bcd
  ld     e,a
  ld     a,l
  add    e			;See above
  daa
  ld     l,a
  jr     nc,+
  ld     a,h
  add    1
  daa
  or     a
  jr     z,win
  ld     h,a
+:

  ld     a,h                    ;(SCORE)=HL
  ld     (SCORE),a
  ld     a,l
  ld     (SCORE+1),a
  ret
  
  
check_win:
  ld     hl,SCORE               ;HL=(SCORE)
  ldi    a,(hl)
  ld     l,(hl)
  ld     h,a

  ld     a,h
  cp     $99
  ret    nz
  ld     a,l
  cp     $99
  ret    nz
win:
  ; Epic win !
  xor    a			;Shuts up the gameroad sound
  ldh    ($17),a
  ld     a,%100			;Don't call enter_pause, just freeze everything
  ld     (PAUSE_FLAG),a
  ld     (WON),a
  ld     de,music_win
  ld     hl,Sot_Music
  rst    $10
  call   setupmusic
  ld     a,Sot_F_Play
  ld     (Sot_Flags),a
  call   waitforsongend
  call   gameover
  inc    sp			;Ret before calling routine (incscore/updscore)
  inc    sp
  ret


;Gameover after score = 9999 or time = 29:59
gameover:
  ;Shuts up the gameroad sound
  xor    a
  ldh    ($17),a
  call   gen_passwd
  ld     de,Setup_gameover
  call   Setup_exit_white
  ret


update_lives:
;----------------------------------------------------------------
;Display remaining lives
  push   af
  call   clear_lives

  ld     hl,$9800+(32*1)+15
  ld     a,(LIVES)
  or     a
  jr     z,+
  ld     b,a
  pop    af
-:
  call   wait_write
  ldi    (hl),a
  dec    b
  jr     nz,-
  ret
+:
  pop    af
  ret

clear_lives:
  ld     hl,$9800+(32*1)+15
  ld     b,4			;MAX_LIVES
  ld     a,c
-:
  call   wait_write
  ldi    (hl),a
  dec    b
  jr     nz,-
  ret



inc_time:
  ld     a,(TIME_MIN)		;Limit to 30 minutes
  cp     $30
  jr     z,+

  ld     a,(TIME_FRAME)
  inc    a
  ld     (TIME_FRAME),a
  cp     60
  jr	 nz,+
  xor    a
  ld     (TIME_FRAME),a


  ld     a,(TIME_SEC)		;Inc seconds
  inc    a
  ld     (TIME_SEC),a
  and    $F
  cp     $A
  jr	 nz,+
  ld     a,(TIME_SEC)
  and    $F0
  add    $10
  ld     (TIME_SEC),a

  and    $F0
  cp     $60
  jr	 nz,+
  xor    a
  ld     (TIME_SEC),a
  ld     a,(TIME_MIN)		;Inc minutes
  inc    a
  ld     (TIME_MIN),a

  and    $F
  cp     $A
  jr	 nz,+
  ld     a,(TIME_MIN)
  and    $F0
  add    $10
  ld     (TIME_MIN),a
+:

  ld     a,(TIME_SEC)
  cp     $59
  ret    nz
  ld     a,(TIME_MIN)
  cp     $29
  ret    nz
  jp     gameover 		; call+ret = jp

;HL = VRAM, DE = min/sec, B = ascii index
disp_time:
  ld     a,d
  swap   a
  and    $F
  add    b
  call   wait_write
  ldi    (hl),a
  ld     a,d
  and    $F
  add    b
  call   wait_write
  ldi    (hl),a
  ld     a,':'-48
  add    b
  call   wait_write
  ldi    (hl),a
  ld     a,e
  swap   a
  and    $F
  add    b
  call   wait_write
  ldi    (hl),a
  ld     a,e
  and    $F
  add    b
  call   wait_write
  ldi    (hl),a
  ret
