; This file is part of the source code for "Super Connard" V1
; Creative Commons BY-NC-SA 2013 - furrtek.org

Setup_world:
  di
  call	 screen_off

  ld     hl,(:tiles_alpha*$4000)+tiles_alpha
  ld     de,$8000
  call   loadtiles_auto

  ld     hl,(:tile_cur*$4000)+tile_cur
  ld     de,$8000+(96*16)
  call   loadtiles_auto

  ld     hl,(:tiles_world*$4000)+tiles_world
  ld     de,$8000+(97*16)
  call   loadtiles_auto

  call   clearbkg
  call   clearsprites
  ld     hl,OAMCOPY
  ld     bc,$A0
  call   clear

  ;We map the world, we map the children
  ld     a,97
  ld     de,$9881
  ld     hl,(:map_world*$4000)+map_world
  call   map

  ld     hl,$9804+(2*32)
  ld     de,text_choice
  ld     b,32
  call   maptext

  ld     de,OAMCOPY
  ld     hl,map_cursor2
  ld     b,4
-:
  ld     a,160                 ;Y
  ld     (de),a
  inc    e
  ld     a,160                 ;X
  ld     (de),a
  inc    e
  ld     a,96                  ;Tile
  ld     (de),a
  inc    e
  ldi    a,(hl)                ;Attributes
  ld     (de),a
  inc    e
  dec    b
  jr     nz,-

  call   startsfx

  xor    a
  ld     (WORLDSEL),a
  ld     (WRITEWAIT),a
  ld     (FADEIDX),a

  call   initwrite_world

  xor    a
  ldh    ($42),a                ;Scroll Y
  ldh    ($43),a                ;Scroll X

  ld     a,%11100100            ;BG palette
  ldh    ($47),a
  ld     a,%11100000            ;SPR0 palette
  ldh    ($48),a
  ld     a,%10010011            ;Turn on screen, BG on, sprites ON 8x8, window OFF, tiles at $8000
  ldh    ($40),a
  ld     a,%00010000            ;Vblank
  ldh    ($41),a
  ld     a,%00000001
  ldh    ($FF),a

  ld     hl,VBL_HANDLER
  ld     de,VBL_world
  ld     (hl),e
  inc    hl
  ld     (hl),d

  xor    a
  ldh    ($0F),a                ;Tell pending interrupts we don't give a shit about them
  ei

  ret
  
text_w_1:
.db "Etats Unis:",$FE,"Joseph Herring",$FE,$FE,"Cogner des",$FE,"animaux sur la",$FE,"route.",$FF
text_w_2:
.db "Allemagne:",$FE,"Adolph Hitler",$FE,$FE,"D{porter des",$FE,"emmerdeurs.",$FF
text_w_3:
.db "Cor{e du Nord:",$FE,"Kim Jong Un",$FE,$FE,"Voler la",$FE,"nourriture",$FE,"du peuple.",$FF

textmap_world:
.dw text_w_1,text_w_2,text_w_3

jt_world:
.dw Setup_introroad,Setup_introcamp,Setup_introkim

initwrite_world:
  ld     hl,$9800+(32*11)+1
  ld     bc,$20*6
  call   clear_w

  ld     hl,textmap_world
  ld     a,(WORLDSEL)
  ld     d,0
  sla    a
  ld     e,a
  add    hl,de
  ldi    a,(hl)
  ld     (WRITETPTR+1),a
  ld     a,(hl)
  ld     (WRITETPTR),a

  ld     de,$9800+(32*11)+1
  ld     hl,WRITEVPTR
  rst    $10
  
  xor    a
  ld     (WRITEWAIT),a
  ret

;----------------------------------------------------------------
;VBL

VBL_world:
  call   fadefromwhite

  call   mapdialog
  call   RAMtoOAM

;----------------------------------------------------------------
;Input handling

  call   readinput

  ld     a,(JOYP_ACTIVE)
  bit    0,a
  jr     z,+                    ;A button
  ld     hl,GAME_FLAGS
  set    7,(hl)			;Flag for new game
  ld     hl,jt_world
  ld     a,(WORLDSEL)
  ld     d,0
  sla    a
  ld     e,a
  add    hl,de
  ldi    a,(hl)			;DE=(jt_world+2*WORLDSEL)
  ld     d,(hl)
  ld     e,a
  ld     hl,RESETX
  res    0,(hl)
  jp     Setup_exit_white	;Call+ret = jp
+:

  ld     a,(JOYP_ACTIVE)
  bit    1,a
  jr     z,+                    ;B button
  ld     de,Setup_title
  jp     Setup_exit_white	;Call+ret = jp
+:

  ld     a,(JOYP_ACTIVE)
  bit    4,a                    ;Right
  jr     z,+
  ld     a,(WORLDSEL)
  cp     3-1
  jr     z,+
  ld     hl,fxscore		;Selection beep
  call   playsfx
  inc    a
  ld     (WORLDSEL),a
  call   initwrite_world
+:

  ld     a,(JOYP_ACTIVE)
  bit    5,a                    ;Left
  jr     z,+
  ld     a,(WORLDSEL)
  or     a
  jr     z,+
  ld     hl,fxscore		;Selection beep
  call   playsfx
  dec    a
  ld     (WORLDSEL),a
  call   initwrite_world
+:

;----------------------------------------------------------------
;Get cursor's position

  ld     hl,curpos_world
  ld     a,(WORLDSEL)
  sla    a
  ld     d,0
  ld     e,a
  add    hl,de
  ldi    a,(hl)
  ld     e,a
  ld     a,(hl)
  ld     d,a

;----------------------------------------------------------------
;OAM buffer update (cursor)

  ld     hl,OAMCOPY

  ld     a,d            ;Sprite 0 (top left)
  ldi    (hl),a         ;Y
  ld     a,e
  ldi    (hl),a         ;X

  add    8              ;X+8
  ld     e,a
  inc    hl
  inc    hl

  ld     a,d            ;Sprite 1 (top right)
  ldi    (hl),a
  add    8              ;Y+8
  ld     d,a
  ld     a,e
  ldi    (hl),a
  sub    8              ;X-8
  ld     e,a
  inc    hl
  inc    hl

  ld     a,d            ;Sprite 2 (bottom left)
  ldi    (hl),a
  ld     a,e
  ldi    (hl),a
  add    8              ;X+8
  ld     e,a
  inc    hl
  inc    hl

  ld     a,d            ;Sprite 3 (bottom right)
  ldi    (hl),a
  ld     a,e
  ld     (hl),a

  ld     hl,VBLC
  inc    (hl)
  ret

map_cursor2:
.db %00000000 ;Attributes (flip)
.db %00100000
.db %01000000
.db %01100000

curpos_world:
.db 32,64
.db 82,58
.db 138,66
